@@ -28,6 +28,7 @@
 #include <sys/stat.h>
 #include <time.h>
 #include <glib.h>
+#include "config.h"
 #include "debug.h"
 #include "libc_internal.h"
 #if GLIB_CHECK_VERSION(2,6,0)
@@ -38,6 +39,26 @@
 #define g_stat stat
 #endif
 
+#ifdef ENABLE_NLS
+#  include <locale.h>
+#  include <libintl.h>
+#  define _(String) ((const char *)dgettext(PACKAGE, String))
+#  ifdef gettext_noop
+#    define N_(String) gettext_noop (String)
+#  else
+#    define N_(String) (String)
+#  endif
+#else
+#  include <locale.h>
+#  define N_(String) (String)
+#  ifndef _
+#    define _(String) ((const char *)String)
+#  endif
+#  define ngettext(Singular, Plural, Number) ((Number == 1) ? ((const char *)Singular) : ((const char *)Plural))
+#  define dngettext(Domain, Singular, Plural, Number) ((Number == 1) ? ((const char *)Singular) : ((const char *)Plural))
+#endif
+
+
 static char errbuf[1024];
 
 /* helpers */
@@ -294,13 +315,32 @@ struct hostent* wpurple_gethostbyname(const char *name) {
 }
 
 /* string.h */
-char* wpurple_strerror( int errornum ) {
-	if( errornum > WSABASEERR ) {
-		sprintf( errbuf, "Windows socket error #%d", errornum );
-		return errbuf;
+char* wpurple_strerror(int errornum) {
+	if (errornum > WSABASEERR) {
+		switch(errornum) {
+			case WSAECONNABORTED: /* 10053 */
+				g_snprintf(errbuf, sizeof(errbuf), _("Connection interrupted by other software on your computer."));
+				break;
+			case WSAECONNRESET: /* 10054 */
+				g_snprintf(errbuf, sizeof(errbuf), _("Remote host closed connection."));
+				break;
+			case WSAETIMEDOUT: /* 10060 */
+				g_snprintf(errbuf, sizeof(errbuf), _("Connection timed out."));
+				break;
+			case WSAECONNREFUSED: /* 10061 */
+				g_snprintf(errbuf, sizeof(errbuf), _("Connection refused."));
+				break;
+			case WSAEADDRINUSE: /* 10048 */
+				g_snprintf(errbuf, sizeof(errbuf), _("Address already in use."));
+				break;
+			default:
+				g_snprintf(errbuf, sizeof(errbuf), "Windows socket error #%d", errornum);
+		}
+	} else {
+		const char *tmp = g_strerror(errornum);
+		g_snprintf(errbuf, sizeof(errbuf), tmp);
 	}
-	else
-		return strerror( errornum );
+	return errbuf;
 }
 
 /* unistd.h */
@@ -331,7 +371,7 @@ int wpurple_read(int fd, void *buf, unsigned int size) {
 		}
 	} else {
 		/* fd is not a socket handle.. pass it off to read */
-		return read(fd, buf, size);
+		return _read(fd, buf, size);
 	}
 }
 
@@ -354,7 +394,7 @@ int wpurple_write(int fd, const void *buf, unsigned int size) {
 	if(wpurple_is_socket(fd))
 		return wpurple_send(fd, buf, size, 0);
 	else
-		return write(fd, buf, size);
+		return _write(fd, buf, size);
 }
 
 int wpurple_recv(int fd, void *buf, size_t len, int flags) {
@@ -382,7 +422,7 @@ int wpurple_close(int fd) {
 			return 0;
 	}
 	else
-		return close(fd);
+		return _close(fd);
 }
 
 int wpurple_gethostname(char *name, size_t size) {
@@ -407,7 +447,7 @@ int wpurple_gettimeofday(struct timeval *p, struct timezone *z) {
 
 	if (p != 0) {
 		_ftime(&timebuffer);
-	   	p->tv_sec = timebuffer.time;			/* seconds since 1-1-1970 */
+		p->tv_sec = timebuffer.time;			/* seconds since 1-1-1970 */
 		p->tv_usec = timebuffer.millitm*1000; 	/* microseconds */
 	}
 
@@ -417,8 +457,21 @@ int wpurple_gettimeofday(struct timeval *p, struct timezone *z) {
 /* stdio.h */
 
 int wpurple_rename (const char *oldname, const char *newname) {
+
+#if GLIB_CHECK_VERSION(2,8,5)
+
+	return g_rename(oldname, newname);
+
+#else
+
+	/* This is a ugly, but we still compile with 2.6.10 but use newer runtimes */
 	struct stat oldstat, newstat;
 
+	/* As of Glib 2.8.5, g_rename() uses MoveFileEx() with MOVEFILE_REPLACE_EXISTING to behave more sanely */
+	if (glib_check_version(2, 8, 5) == NULL) {
+		return g_rename(oldname, newname);
+	}
+
 	if(g_stat(oldname, &oldstat) == 0) {
 		/* newname exists */
 		if(g_stat(newname, &newstat) == 0) {
@@ -461,6 +514,8 @@ int wpurple_rename (const char *oldname, const char *newname) {
 		return -1;
 	}
 
+#endif
+
 }
 
 /* time.h */
@@ -1038,7 +1093,6 @@ wpurple_get_timezone_abbreviation(const struct tm *tm)
 	return "";
 }
 
-#if !GLIB_CHECK_VERSION(2,8,0)
 /**
  * g_access:
  * @filename: a pathname in the GLib file name encoding (UTF-8 on Windows)
@@ -1063,6 +1117,12 @@ int
 wpurple_g_access (const gchar *filename,
 	  int          mode)
 {
+#if GLIB_CHECK_VERSION(2,8,0)
+
+	return g_access(filename, mode);
+
+#else
+
   if (G_WIN32_HAVE_WIDECHAR_API ())
     {
       wchar_t *wfilename = g_utf8_to_utf16 (filename, -1, NULL, NULL, NULL);
@@ -1103,7 +1163,8 @@ wpurple_g_access (const gchar *filename,
       errno = save_errno;
       return retval;
     }
-}
+
 #endif
+}
 
 
